#!/usr/bin/python
# -*- coding:utf-8 -*-
# Email:iamfengdy@126.com
# DateTime:2024/1/2
# Tool:PyCharm

"""  """
__version__ = '0.0.1'
__history__ = """"""
__all__ = ['read_file', 'get_or_raise']

import os


def read_file(file_path):
    """ 读取文件并设置环境变量 """
    with open(file_path, 'r', encoding='utf-8') as fd:
        for line in fd:
            if 'export' not in line:
                continue
            line = line.replace('\r', '').replace('\n', '')
            _, kv = line.split(maxsplit=1)
            key, value = kv.split('=')
            if value.startswith('$'):
                value = os.environ[value[1:]]
            if value[0] == '"':
                value = value[1:]
            if value[-1] == '"':
                value = value[:-1]
            os.environ[key] = value


def get_or_raise(key, default=None, prefix=None):
    """
    读取指定环境变量或抛出异常
    :param str key:
    :param object default: 默认值
    :param str prefix: 变量前缀，默认为None
    :return: 环境变量值
    """
    if prefix is not None:
        key = prefix + key
    if default is None:
        value = os.environ.get(key)
    else:
        value = os.environ.get(key, default)
    if value is None:
        raise RuntimeError(
            (
                'Environment variable "{}" not found, you must set this variable to run this application.'
            ).format(key)
        )
    return value



#!/usr/bin/env python
# -*-coding:utf-8-*-
# Email:iamfengdy@126.com
# DateTime:2021/08/30 17:24

""" FTP """
import logging
import os
from ftplib import FTP as _FTP
__version__ = '1.0'
__history__ = ''' '''
__all__ = ['FTP']

from shortcut_util import logger


class FTP:
    __slots__ = ['ftp']

    def __init__(self, ftp):
        self.ftp = ftp

    @staticmethod
    def connect(**kwargs):
        logger.info('Connect FTP. kwargs=%s.' % kwargs)
        ftp = _FTP(**kwargs)
        ftp.encoding = 'utf-8'
        ftp.set_debuglevel(0)
        return FTP(ftp)

    def close(self):
        logger.info('Close FTP.')
        if self.ftp is not None:
            self.ftp.close()
            self.ftp = None

    def list_dir(self, path):
        # TODO(fengdy): 如果文件很多会很慢
        # FIXME(fengdy): 文件名称不能包含空格
        logger.info(f'list dir. path={path}')
        infos = []
        files = []
        folders = []
        # FIXME(fengdy): 如果目录不存在会出错
        self.ftp.dir(path, infos.append)
        for info in infos:
            # info示例："drwxr-xr-x 1 ftp ftp 0 Mar 06 13:59 a"
            _, _name = info.rsplit(' ', 1)
            if _name in ('.', ):
                continue
            ff_type = info[0]
            if ff_type == 'd':
                folders.append(_name)
            else:
                files.append(_name)
        return files, folders

    def create_folder(self, path):
        logger.info(f'create folder. path={path}')
        assert os.path.isabs(path), 'must be abs path'
        folders = path.split('/')
        dir_path = '/'
        for folder in folders:
            tmp_path = os.path.join(dir_path, folder)
            if not self.exist_folder(tmp_path):
                self.ftp.cwd(dir_path)
                self.ftp.mkd(folder)
            dir_path = tmp_path

    def exist_folder(self, path):
        try:
            self.ftp.cwd(path)
            return True
        except Exception as e:
            return False

    def exist_file(self, path):
        dir_path, file_name = os.path.split(path)
        if self.exist_folder(dir_path):
            files, _ = self.list_dir(dir_path)
            return file_name in files
        return False

    def __upload_file(self, src_path, dst_path, cover=False):
        """ 上传文件（这里不创建文件夹）

        :param str src_path: 文件源路径，必须是绝对路径
        :param str dst_path: 文件目标路径，必须是绝对路径
        :param bool cover: 是否覆盖，默认False
        :return:
        """
        logger.debug(
            f'upload file. src_path={src_path}. dst_path=%{dst_path}. cover={cover}')
        self.ftp.cwd(dst_path)
        file_name = os.path.basename(src_path)
        self.ftp.storbinary('STOR {0}'.format(
            file_name), open(src_path, 'rb'))

    def __upload_folder(self, src_path, dst_path, cover=False):
        """ 上传文件夹（不上传.开头的文件即隐藏文件）

        :param str src_path: 源路径，必须为绝对路径
        :param str dst_path: 目标路径，比如为绝对路径
        :param bool cover: 是否覆盖，默认False
        """
        logger.debug(
            f'upload folder. source_path={src_path}. dst_path={dst_path}. cover={cover}')
        self.create_folder(dst_path)
        ffs = os.listdir(src_path)
        # files_or_folders
        for ff in ffs:
            if ff.startswith('.'):
                continue
            ff_path = os.path.join(src_path, ff)
            if os.path.isfile(ff_path):
                self.__upload_file(ff_path, dst_path)
            elif os.path.isdir(ff_path):
                self.__upload_folder(ff_path, os.path.join(dst_path, ff))

    def upload(self, src_path, dst_path, cover=False):
        """ 上传文件或文件夹

        :param str src_path: 源路径，必须为绝对路径，如果以”/“结尾，则上传该目录下的内容
        :param str dst_path: 目标路径，比如为绝对路径
        :param bool cover: 是否覆盖，默认False

        ::

            source_path directory structure：'/a/b/c/{1.txt,2.txt,...}'
            upload('/a/b/c', '/t/e')
                => /t/e/c/{1.txt,2.txt,...}
            upload('/a/b/c/', '/t/e')
                => /t/e/{1.txt,2.txt,...}
        """
        # FIXME(fengdy):怎样保证事务
        logger.info(
            f'upload. src_path={src_path}. dst_path={dst_path}. cover={cover}')
        assert os.path.isabs(src_path), 'Source path must be an absolute path!'
        assert os.path.isabs(dst_path), 'Target path must be an absolute path!'
        if not os.path.exists(src_path):
            return
        if src_path.endswith('/'):
            self.__upload_folder(src_path, dst_path, cover)
        else:
            # upload file or folder
            if os.path.isfile(src_path):
                self.__upload_file(src_path, dst_path, cover)
            elif os.path.isdir(src_path):
                _basename = os.path.basename(src_path)
                _dst_path = os.path.join(dst_path, _basename)
                self.__upload_folder(src_path, _dst_path, cover)
            else:
                logger.warning(
                    'Source path type is invalid. path=%s.' % src_path)

    def __download_file(self, src_path, dst_path, cover=False):
        logger.debug(
            f'download file. src_path={src_path}. dst_path={dst_path}. cover={cover}')
        dir_path, file_name = os.path.split(src_path)
        dst_file_path = os.path.join(dst_path, file_name)
        self.ftp.cwd(dir_path)
        self.ftp.retrbinary('RETR {0}'.format(
            file_name), open(dst_file_path, 'wb').write)

    def __download_folder(self, src_path, dst_path, cover=False):
        logger.debug(
            f'download folder. src_path={src_path}. dst_path={dst_path}. cover={cover}')
        os.path.makedirs(dst_path)
        files, folders = self.list_dir(src_path)
        for _file in files:
            _src_path = os.path.join(src_path, _file)
            self.__download_file(_src_path, dst_path)
        for _folder in folders:
            _src_path = os.path.join(src_path, _folder)
            _dst_path = os.path.join(dst_path, _folder)
            self.__download_folder(_src_path, _dst_path)

    def download(self, src_path, dst_path, cover=False):
        logger.debug(
            f'Download folder. src_path={src_path}. dst_path={dst_path}. cover={cover}')
        assert os.path.isabs(src_path), 'Source path must be an absolute path!'
        assert os.path.isabs(dst_path), 'Target path must be an absolute path!'
        if src_path.endswith('/'):
            self.__download_folder(src_path, dst_path, cover)
        else:
            if self.exist_file(src_path):
                self.__download_file(src_path, dst_path, cover)
            elif self.exist_folder(src_path):
                _basename = os.path.basename(src_path)
                _dst_path = os.path.join(dst_path, _basename)
                self.__download_folder(src_path, _dst_path, cover)



#!/usr/bin/env python
# -*-coding:utf-8-*-
# @Email:iamfengdy@126.com
# @DateTime:2021/07/17 15:53

""" 单例模式 """
__version__ = '1.0'
__history__ = ''' '''
__all__ = []


def singleton(func):
    instances = {}

    def _func(*args, **kwargs):
        if func not in instances:
            instances[func] = func(*args, **kwargs)
        return instances[func]
    return _func



#!/usr/bin/python
# -*- coding:utf-8 -*-
# Email:iamfengdy@126.com
# DateTime:2023/11/15
# Tool:PyCharm

"""  """
__version__ = '0.0.1'
__history__ = """"""
__all__ = ['uuid_id', 'ts_id', 'datetime_id']

import string
import time
import uuid
import random
from datetime import datetime


def uuid_id():
    return uuid.uuid3(uuid.uuid1(), uuid.uuid4().hex).hex


def make_salt(length):
    salt = str(
        ''.join(
            random.sample(
                string.ascii_letters +
                string.digits,
                length))).upper()
    return salt


def ts_id(salt_length=6):
    ts = str(int(time.time()))
    salt = make_salt(length=salt_length)
    return f'{ts}_{salt}' if salt else ts


def datetime_id(salt_length=6, fmt='%Y%m%d%H%M%S'):
    dt = datetime.now().strftime(fmt)
    salt = make_salt(length=salt_length)
    return f'{dt}_{salt}' if salt else dt



#!/usr/bin/env python
# -*-coding:utf-8-*-
# Email:iamfengdy@126.com
# DateTime:2021/08/30 11:33

""" 装饰器 """
__version__ = '1.0'
__history__ = ''' '''
__all__ = ['catch_exception']

from functools import wraps

from shortcut_util import logger


def catch_exception(func):
    """ 捕获异常，返回默认值（如果存在kwargs['default']）

    :param func: 方法
    :return: 返回值或默认值
    """
    @wraps(func)
    def _func(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except Exception as exc:
            logger.exception(exc)
            result = kwargs.get('default', None)
        finally:
            return result
    return _func



#!/usr/bin/env python
# -*-coding:utf-8-*-
# Email:iamfengdy@126.com
# DateTime:2021/08/30 13:33

""" Email """
import os.path
from email.mime.nonmultipart import MIMENonMultipart
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage
from email.mime.audio import MIMEAudio
from email.mime.application import MIMEApplication
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from smtplib import SMTP
__version__ = '1.0'
__history__ = ''' '''
# __all__ = []


class TextEmail(MIMEText):
    """Plain/text"""

    def __init__(self, _text, **kwargs):
        super(TextEmail, self).__init__(_text,  _charset="utf-8", **kwargs)


# other mime type: binary（MIMEApplication）、image（MIMEImage）、audio（MIMEAudio）、（MIMENonMultipart）


class HtmlEmail(MIMEMultipart):
    def __init__(self, **kwargs):
        super(HtmlEmail, self).__init__(**kwargs)


def test_html_email():
    """Present as HTML.
    Resources in HTML are not shown in attachments.
    """
    email = HtmlEmail(_subtype='alternative')
    # add body
    _mime = MIMEText(
        '<p>html</p><p><img src="cid:image_id"></p>', 'html', 'utf-8')
    email.attach(_mime)
    # add attachment
    file_path = '../tests/test.png'
    _mime = MIMEImage(open(file_path, 'rb').read())
    _mime.add_header("Content-ID", "<image_id>")
    email.attach(_mime)
    return email


def test_html_email_with_attachment():
    """present as HTML.
    Resources in HTML are not shown in attachments.
    """
    email = HtmlEmail(_subtype='related')
    # add body
    _mime = MIMEText(
        '<p>html with attachment</p><p><img src="cid:image_id"></p>', 'html', 'utf-8')
    email.attach(_mime)
    # attachment1
    file_path = '../tests/test.png'
    _mime = MIMEImage(open(file_path, 'rb').read())
    _mime.add_header("Content-ID", "<image_id>")
    file_name = os.path.basename(file_path)
    _mime.add_header("Content-Disposition", "attachment", filename=file_name)
    email.attach(_mime)
    # attachment2
    file_path = 'cache_util.py'
    _mime = MIMEText(open(file_path, 'r').read())
    file_name = os.path.basename(file_path)
    # present the file_name as an attachment
    _mime.add_header("Content-Disposition", "attachment", filename=file_name)
    email.attach(_mime)
    return email


class EmailWithAttachment(MIMEMultipart):
    def __init__(self, **kwargs):
        super(EmailWithAttach, self).__init__(**kwargs)


def test_email_with_attachment():
    email = EmailWithAttach()
    # add body
    _mime = MIMEText('body is text.', _charset='utf-8')
    email.attach(_mime)
    # add attachment
    # select MIME types depend on attachment type
    file_path = 'cache_util.py'
    _mime = MIMEText(open(file_path, 'r').read())
    file_name = os.path.basename(file_path)
    # present the file_name as an attachment
    _mime.add_header("Content-Disposition", "attachment", filename=file_name)
    email.attach(_mime)
    return email


class Email:
    def __init__(self, smtp, from_user):
        assert isinstance(smtp, SMTP), 'Not SMTP instance!'
        self.smtp = smtp
        self.from_user = from_user
        pass

    @staticmethod
    def connect(host, port, user, password):
        smtp = SMTP()
        try:
            smtp.connect(host, port)
            smtp.login(user, password)
            return Email(smtp, user)
        except Exception as e:
            raise e

    def close(self):
        if self.smtp is not None:
            self.smtp.quit()
            self.smtp.close()
            self.smtp = None

    def send(self, email, to_users, cc_users, subject):
        assert isinstance(email, MIMEBase), 'Not MIMEBase!'

        email['From'] = self.from_user
        email['To'] = to_users
        email['Cc'] = cc_users
        email['Subject'] = subject
        self.smtp.sendmail(self.from_user, to_users, email.as_string())



#!/usr/bin/env python
# -*-coding:utf-8-*-
# Email:iamfengdy@126.com
# DateTime:2021/08/30 11:08

""" 常量 """
__version__ = '1.0'
__history__ = ''' '''
__all__ = ['Const']


class Const(object):
    __slots__ = ['__upload_interval']

    def __init__(self, **kwargs):
        # 使用这种方式命令可以实例类就不能直接访问了
        self.__upload_interval = 30
        pass

    @property
    def upload_interval(self):
        return self.__upload_interval

    @upload_interval.setter
    def upload_interval(self, value):
        # TODO: valid
        self.__upload_interval = value

    @upload_interval.deleter
    def upload_interval(self):
        del self.__upload_interval



#!/usr/bin/env python
# -*-coding:utf-8-*-
# @Email:iamfengdy@126.com
# @DateTime:2021/07/17 10:57

""" 日志 """
__version__ = '1.0'
__history__ = ''' '''
__all__ = ['TimedRotatingFileHandler', 'configure_logging']

import os
import time
from stat import ST_MTIME
import logging
import logging.config
from logging.handlers import TimedRotatingFileHandler as _TimedRotatingFileHandler


class TimedRotatingFileHandler(_TimedRotatingFileHandler):

    def __init__(self, filename, **kwargs):
        _TimedRotatingFileHandler.__init__(self, filename, **kwargs)
        self._filename = filename
        self.is_first = True
        # self.rolloverAt = self.computeRollover(t)

    def shouldRollover(self, record):
        """
        增加每次执行时对日志是否需要分割的检测，需要注意日级与其他类型不同，主要是时间范围不同；
        """
        result = False
        ts = int(time.time())
        if os.path.isfile(self._filename):
            mts = os.stat(self._filename)[ST_MTIME]
        else:
            mts = ts
        _rolloverAt = self.rolloverAt - self.interval
        if self.is_first and mts < _rolloverAt:
            self.rolloverAt = _rolloverAt
            result = True
        if ts >= self.rolloverAt:
            result = True
        self.is_first = False
        return result


# 日志配置可以参考`django.utils.log`以及`logging`
DEFAULT_LOGGING_CLASS = 'utils.log.TimedRotatingFileHandler'
DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {},
    'formatters': {
        'default': {
            "format": (
                "[%(asctime)s] %(pathname)s "
                "%(lineno)d %(process)d %(thread)d "
                "%(levelname)s %(message)s"
            ),
            "datefmt": "%Y-%m-%d %H:%M:%S",
        }
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            "formatter": "default",
        },
        'root': {
            "class": DEFAULT_LOGGING_CLASS,
            "formatter": "default",
            "filename": "result.log",
            # "maxBytes": 1024 * 1024 * 10,
            "backupCount": 10,
        }
    },
    'loggers': {
        'root': {
            'handlers': ['root'],
            'level': 'INFO',
        },
        'console': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
    }
}


def configure_logging(logging_settings=None):
    logging_settings = logging_settings or DEFAULT_LOGGING
    logging.config.dictConfig(logging_settings)



#!/usr/bin/env python3
import os
import re
import shutil
import sys
from io import open

from setuptools import find_packages, setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 10)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================

This version requires Python {}.{}, but you're trying
to install it on Python {}.{}.

This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have pip >= 9.0 and setuptools >= 24.2, then try again:

    $ python -m pip install --upgrade pip setuptools
    $ python -m pip install shortcut-util

""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


def read(f):
    return open(f, 'r', encoding='utf-8').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('shortcut_util')


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    if os.system("twine check dist/*"):
        print("twine check failed. Packages might be outdated.")
        print("Try using `pip install -U twine wheel`.\nExiting.")
        sys.exit()
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree('shortcut-util.egg-info')
    sys.exit()


setup(
    name='shortcut-util',
    version=version,
    url='https://gitee.com/iamfengdy/shortcut-util',
    license='BSD',
    description='Common Utils for Python, made easy.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='fengdy',
    author_email='iamfengdy@126.com',  # SEE NOTE BELOW (*)
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    keywords='shortcut util',
    install_requires=[],
    python_requires=">=3.10",
    zip_safe=False,
    classifiers=[   # https://pypi.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Adaptive Technologies',
    ],
    project_urls={
        'Source': 'https://gitee.com/iamfengdy/shortcut-util',
    },
)



#!/usr/bin/python
# -*- coding:utf-8 -*-
# Email:iamfengdy@126.com
# DateTime:2024/1/18
# Tool:PyCharm

"""  """
__version__ = '0.0.1'
__history__ = """"""
__all__ = ['word2pdf']

import retry as retry


@retry(tries=3, delay=1)
def word2pdf(file_path):
    """
    https://github.com/unoconv/unoconv

    cd /tmp && wget -l https://gist.githubusercontent.com/regebro/036da022dc7d5241a0ee97efdf1458eb/raw/1bc0655423d196acd79a5d9fa60d2baada8dd534/find_uno.py && python3 find_uno.py

    apt-get install libreoffice unoconv

    如果遇到找不到的问题，通过find_uno.py查找所支持的python，然后调整unoconv(/usr/bin/unoconv)

    :param str file_path: 文件路径
    :return: str
    """
    import subprocess
    subprocess.run(['unoconv', '--format=pdf', file_path], check=True)
    # subprocess.CalledProcessError



#!/usr/bin/env python
# -*-coding:utf-8-*-
# @Email:iamfengdy@126.com
# @DateTime:2021/07/17 15:42

""" 时间 """
__version__ = '1.0'
__history__ = ''' '''
__all__ = ['ts2str', 'str2ts']

import time
from datetime import datetime, timedelta


def ts2str(ts=None, fmt='%Y-%m-%d %H:%M:%S', utc=8):
    ts = ts or int(time.time())
    _tt = time.gmtime(ts + utc * 3600)
    return time.strftime(fmt, _tt)


def str2ts(dt_str, fmt='%Y-%m-%d %H:%M:%S'):
    tm = time.strptime(dt_str, fmt)
    ts = int(time.mktime(tm))
    return ts


def utc2local(utc_str, utc_fmt):
    local_str = datetime.strptime(utc_str, utc_fmt) + timedelta(hours=8)
    return str(local_str)
